/*
 * mod1535_ton_toff.c
 *
 * Created: 2014-05-29 12:23:11
 *  Author: dsosnowski
 */ 


#define F_CPU 8000000UL
#include <avr/io.h>
#include <avr/interrupt.h>
#include <inttypes.h>
#include <util/delay.h>
#include <avr/wdt.h>
#include <avr/sleep.h>
#include <avr/pgmspace.h>
#include "ee_mem.h"
#include "wdog.h"
#include "defs.h"

//0...9, 0, -, null
static char digits[15] = {0x42, 0xEE, 0x92, 0x86, 0x2E, 0x07, 0x03, 0xCE, 0x02, 0x06, 0xEC, 0xBF, 0xFF, 0xDF, 0xF7};

#define NULL_INDX	12

//-------------------------
void HardInit(void);
void Display(char val);
void SwitchScan(void);
//void Config2Scan(void);
void SettRestore(void);
void ConfigMode(void);

struct {
	char chars[2];
	char dot[2];
} display;

struct {
	volatile char tick;
	char seconds;
	char action;
	char mode;
	char ton;
	char toff;
} system;

struct {
	char enable;
	char mode;
	char ton;
	char toff;
} config;


int main(void)
{
	char i = 0;
	HardInit();
	_delay_ms(100);
	SettRestore();    
	while(1)
    {
	
		SwitchScan();

		//--------------------------------------------------------
		//normal mode
		if (config.mode == 0)
		{
			if (GET_CFG == 0)
			{
				if (system.tick)
				{
					if (system.seconds)
					{
						system.seconds--;
						system.tick = 0;
					}
					else
						system.seconds = 59;					
				}
				if (system.seconds & 1) 
					display.dot[0] = 1;
				else
					display.dot[0] = 0;
			}
			else
				display.dot[0] = 0;

			
			if (system.tick)
			{								
				system.tick = 0;

				if (system.mode == 0)
				{
					if (system.toff) system.toff--;
					if (system.toff == 0)
					{
						system.ton = config.ton;
						system.mode = 1;
					}
				}
				else
				{
					if (system.ton) system.ton--;
					if (system.ton == 0)
					{
						system.toff = config.toff;
						system.mode = 0;
					}
				}
				
				
				if (system.mode == 0)
				{
					OUT_OFF;
					Display(system.toff);
					display.dot[1] = 0;
				}
				else
				{
					OUT_SET;
					Display(system.ton);
					display.dot[1] = 1;
				}	
			}
		}
		
		//--------------------------------------------------
		//config mode
		else
		{
			ConfigMode();	
		}

		//--------------------------------------------------
		_delay_ms(30);
		WdtReset();
    }
}


//-----------------------------------------------------------
void HardInit(void)
{
	//UI_PORT = (1<<ANODE1 | 1<<ANODE2 | 1<<SW_PIN | 1<<IN_PIN | 1<<CF1_PIN | 1<<CF2_PIN);
	UI_PORT = (1<<SW1_PIN | 1<<SW2_PIN | 1<<CFG_PIN);
	UI_DDR = (1<<ANODE1 | 1<<ANODE2 | 1<<RELAY_PIN);
	
	SEG_PORT = 0x00;
	SEG_DDR = 0xFF;

	//for display
	TCCR0 = (1<<PSR0 | 1<<CS01);
	TIMSK |= (1<<TOIE0);

	//for tick
	TCCR1A = 0;
	TCCR1B = (1<<CTC1 | 1<<PSR1 | 1<<CS13 | 1<<CS12 | 1<<CS11 | 1<<CS10);		//div 16384
	OCR1C = TICK_0_5S;
	TIMSK |= (1<<TOIE1);

	WdtSet(7);

	sei();
}

//-----------------------------------------------------------
//Display
SIGNAL(TIMER0_OVF0_vect)
{
	static char anode;
	if (anode < ANODES_FACTOR) anode++;
	else anode = 0;
	
	switch (anode)
	{
		case 0:
			ANODES_PORT |= (1<<ANODE2);
			SEG_PORT = display.chars[0];
			if (display.dot[0]) DP_SET;
			ANODES_PORT &= ~(1<<ANODE1);
		break;
		
		case 1:
			ANODES_PORT |= (1<<ANODE1);
			SEG_PORT = display.chars[1];
			if (display.dot[1]) DP_SET;
			ANODES_PORT &= ~(1<<ANODE2);
		break;

		case 2:
		ANODES_PORT |= (1<<ANODE2);
		//SEG_PORT = display.chars[1];
		//ANODES_PORT &= ~(1<<ANODE2);
		break;
	}
}

//-----------------------------------------------------------
//Tick
SIGNAL(TIMER1_OVF1_vect)
{
	static char div;
	div++;
	if (div >= 2)
	{
		div = 0;
		system.tick = 1;
	}
}

//-----------------------------------------------------------
void Display(char val)
{
	char i;
	i = (val / 10);
	if (i > 10) i = 0;
	display.chars[0] = digits[i];
	
	i = (val % 10);
	display.chars[1] = digits[i];
}

//-----------------------------------------------------------
void SwitchScan(void)
{
	static char swpress1;
	static char swpress2;
	system.action = 0;
	if (GET_SW2 == 0)
	{
		if (swpress1 < 0xFF) swpress1++;
		if (swpress1 == SWITCH_HOLD) 
		{
			if (config.mode == 0) 
				config.mode = 1;							//enter prog mode
			else
				system.action = 3;
		}		
	}
	else
	{
		if (swpress1 >= SWITCH_HOLD)  swpress1 = 0;		
		if (swpress1 >= SWITCH_PRESS) system.action = 1;	//step down
		swpress1 = 0;
	}

	if (GET_SW1 == 0)
	{
		if (swpress2 < 0xFF) swpress2++;
		if (swpress2 == SWITCH_HOLD)
		{
			if (config.mode == 0)
				config.mode = 3;								//enter prog mode
			else
				system.action = 3;
		}
	}
	else
	{
		if (swpress2 >= SWITCH_HOLD)  swpress2 = 0;
		if (swpress2 >= SWITCH_PRESS) system.action = 2;	//step up
		swpress2 = 0;
	}
}

//-----------------------------------------------------------
void SettRestore(void)
{
	config.ton = ee_read(5);
	config.toff = ee_read(6);
	if (config.ton > 99) config.ton = 1;
	if (config.ton == 0) config.ton = 1;
	if (config.toff > 99) config.toff = 1;
	if (config.toff == 0) config.toff = 1;
}

//-----------------------------------------------------------
void ConfigMode(void)
{
	static char blink;
	char di1;
	char di2;
	char i;
	
	if (config.mode == 1)				//first step
	{
		di1 = digits[13];
		di2 = digits[13];
		
		if ((system.action == 1) || (system.action == 2))
		{	
			config.mode = 2;			//next
			system.action = 0;
		}
		if (system.action == 3)
		{
			config.mode = 6;			//exit
			system.action = 0;
		}
	}

	if (config.mode == 2)				
	{
		if (system.action == 1)
		{
			config.ton++;				
			if (config.ton > 99) config.ton = 1; 
		}
		if (system.action == 2)
		{
			config.ton--;
			if (config.ton == 0) config.ton = 99;
		}
		i = (config.ton / 10);
		di1 = digits[i];
			
		i = (config.ton % 10);
		di2 = digits[i];
		
		if (system.action == 3)
		{
			config.mode = 5;			//save
		}
	}

	if (config.mode == 3)				
	{
		di1 = digits[14];
		di2 = digits[14];
		if ((system.action == 1) || (system.action == 2))
		{
			config.mode = 4;			//next
			system.action = 0;
		}
		if (system.action == 3)
		{
			config.mode = 6;			//exit
			system.action = 0;
		}
	}

	if (config.mode == 4)				
	{
		if (system.action == 1)
		{
			config.toff++;
			if (config.toff > 99) config.toff = 1;
		}
		if (system.action == 2)
		{
			config.toff--;
			if (config.toff == 0) config.toff = 99;
		}
		i = (config.toff / 10);
		di1 = digits[i];
		
		i = (config.toff % 10);
		di2 = digits[i];

		if (system.action == 3)
		{
			config.mode = 5;			//save
		}
	}
	
	system.action = 0;

	if (config.mode == 5)				
	{
		di1 = digits[12];
		di2 = digits[12];
		display.chars[0] = digits[12];
		display.chars[1] = digits[12];
		cli();
		SEG_PORT = 0xFF;
		ee_write(5, config.ton);
		ee_write(6, config.toff);
		system.ton = 0;
		system.toff = 0;
		system.mode = 0;
		OUT_OFF;
		system.seconds = 0;
		system.tick = 0;
		config.mode = 0;	
		_delay_ms(500);
		sei();		
	}

	if (config.mode > 5) 
	{
		system.ton = 0;
		system.toff = 0;
		system.mode = 0;
		OUT_OFF;
		system.seconds = 0;
		system.tick = 0;
		config.mode = 0;		
	}
	
	
	if (blink < BLINK_OFF) 
		blink++;
	else 
		blink = 0;
	
		if (blink < BLINK_ON)
		{
			display.chars[0] = di1;
			display.chars[1] = di2;
		}
		else
		{
			display.chars[0] = digits[12];
			display.chars[1] = digits[12];
		}
}